@@ -69,6 +69,8 @@ static void                 garcon_gtk_menu_add                         (GarconG
                                                                          GarconMenu              *garcon_menu);
 static void                 garcon_gtk_menu_load                        (GarconGtkMenu           *menu);
 static void                 garcon_gtk_menu_reload                      (GarconGtkMenu           *menu);
+static void                 garcon_gtk_menu_load_cancel                 (gpointer                 data,
+                                                                         GObject                 *garcon_menu);
 
 
 
@@ -240,7 +242,10 @@ garcon_gtk_menu_finalize (GObject *object)
 
   /* Release menu */
   if (menu->priv->menu != NULL)
-    g_object_unref (menu->priv->menu);
+    {
+      g_object_weak_unref (G_OBJECT (menu->priv->menu), garcon_gtk_menu_load_cancel, menu);
+      g_object_unref (menu->priv->menu);
+    }
 
   (*G_OBJECT_CLASS (garcon_gtk_menu_parent_class)->finalize) (object);
 }
@@ -592,6 +597,9 @@ garcon_gtk_menu_load_finish (GObject      *source_object,
 
       g_error_free (error);
     }
+  else if (user_data != menu->priv->menu)
+    menu->priv->is_loaded = FALSE;
+
 
   /* destroy all GtkMenu items */
   children = gtk_container_get_children (GTK_CONTAINER (menu));
@@ -619,7 +627,11 @@ garcon_gtk_menu_load_async (GTask        *task,
 
   g_mutex_lock (&menu->priv->load_lock);
 
-  menu->priv->is_loaded = garcon_menu_load (menu->priv->menu, cancellable, &error);
+  if (g_cancellable_set_error_if_cancelled (cancellable, &error))
+    menu->priv->is_loaded = FALSE;
+  else
+    menu->priv->is_loaded = garcon_menu_load (menu->priv->menu, cancellable, &error);
+
   if (! menu->priv->is_loaded)
     g_task_return_error (task, error);
 
@@ -642,7 +654,7 @@ garcon_gtk_menu_load (GarconGtkMenu *menu)
 
   menu->priv->load_cancel = g_cancellable_new ();
   menu->priv->load_task = g_task_new (menu, menu->priv->load_cancel,
-                                      garcon_gtk_menu_load_finish, NULL);
+                                      garcon_gtk_menu_load_finish, menu->priv->menu);
   g_signal_connect_swapped (menu->priv->load_task, "notify::completed",
                             G_CALLBACK (garcon_gtk_menu_reset_load_task), menu);
   g_task_run_in_thread (menu->priv->load_task, garcon_gtk_menu_load_async);
@@ -1091,6 +1103,16 @@ garcon_gtk_menu_new (GarconMenu *garcon_menu)
 
 
 
+static void
+garcon_gtk_menu_load_cancel (gpointer data,
+                             GObject *garcon_menu)
+{
+  GarconGtkMenu *menu = data;
+  g_cancellable_cancel (menu->priv->load_cancel);
+}
+
+
+
 /**
  * garcon_gtk_menu_set_menu:
  * @menu  : A #GarconGtkMenu
@@ -1116,6 +1138,7 @@ garcon_gtk_menu_set_menu (GarconGtkMenu *menu,
   if (garcon_menu != NULL)
     {
       menu->priv->menu = g_object_ref (garcon_menu);
+      g_object_weak_ref (G_OBJECT (menu->priv->menu), garcon_gtk_menu_load_cancel, menu);
       g_signal_connect_object (menu->priv->menu, "reload-required",
                                G_CALLBACK (garcon_gtk_menu_reload), menu, G_CONNECT_SWAPPED);
     }
